function [y1,y2]=CreateData(u1h,u2h,n,t,gamma,exsteps,intsteps,h,np,observation_type)
% Input u1,u2 is initial condition  in Fourier space
% Input gamma is noise variance eg gamma=.01;
% h is time step for integration
% intsteps is number of time points for integration between observations
% exsteps is number of dicrete times points U(t) is observed
% np is reduction factor for euler grid or half length of box around 0 on fourier grid
% L=2*pi, size of torus (XXXX to change at some point)

% global X Y av bv mm w1 w2 w3 w4 g1h g2h c p1 p2 p3;


y1=cell(exsteps,1);
y2=cell(exsteps,1);

for j1=1:exsteps % steps till next datapoint
    for j2=1:intsteps % intermediate integration steps
        [u1h,u2h]=ns2dstepaa(u1h,u2h,n,h);
        t=t+h;
    end
    
    switch lower(observation_type)
        case 'euler'
            
            % time domain ifft
            u1=ifft2(u1h)*n*n;
            u2=ifft2(u2h)*n*n;
            
            u1r=red2d(u1,n,np);
            u2r=red2d(u2,n,np);
            
            noise1=randn(size(u1r));
            noise2=randn(size(u2r));
            
            y1{j1}=real(u1r)+ sqrt(gamma)*noise1;
            y2{j1}=real(u2r)+ sqrt(gamma)*noise2;
        case 'fourier'
            
            u1=ifftshift(u1h);            
            u2=ifftshift(u2h);
            
            x_dim=size(u1,1);
            
            center_0=x_dim/2+1;
            start=center_0-np;
            fin=center_0+np;
            
            u1r=u1(start:fin,start:fin);
            u2r=u2(start:fin,start:fin);
            
            noise1=randn(size(u1r));
            noise2=randn(size(u2r));
            
            y1{j1}=real(u1r)+ sqrt(gamma)*noise1;
            y2{j1}=real(u2r)+ sqrt(gamma)*noise2;
            
            
    end
    
end

end


