% Navier--Stokes in a 2D torus via FFT with initial condition drawn from
% Gaussian prior.
% 
% ACKNOWLEDGMENTS for reusing bits of sample code and ideas:
% 
% Kody Law (personal communication)
% 'Evaluating algorithms for Data Assimilation'
% 
% Hannes Uecker (online)
% 'A short ad hoc introduction to spectral methods for
% parabolic PDE and the Navier--Stokes equations'
% 
% Time-stepping in ns2dstepaa.m, using an anti-aliased product aap2.
% Auxiliary functions: vof.m
% Some global variables defined for convenience.
% 
% N. Kantas
% -----------------------------------------------------------------------

clear
tic 

% SEED=str2num('100')
% rand('twister',SEED)
% randn('state',2^32*rand)


% set to 1 to plot stuff while running the code
want_plot=1;want_video=0;

% Use each np-th point in quiver plots;
np=2; 

global X Y av bv mm w1 w2 w3 w4 p1 p2 p3 c g1h g2h regime KX KY L integration_method;

n=64;

L=2*pi;

dx=L/n; dy=dx;


integration_method='etd_1';
% regime='stationary';
regime='test';
switch lower(regime)
    case 'stationary'
        rey=10; % 1/nu
        nu=1/rey;

        % initial/final time
        T0=0;
        Tf=40;

        % initialise time
        t=T0;
            
        % euler step         
        h=.05;
        % inter observation time         
        dt_obs=.2;
        % % % number of steps  with plotting
        exsteps=(Tf-T0)/dt_obs;
        % % % number of steps between plotting
        intsteps=dt_obs/h;%ceil((Tf-T0)/exsteps/h);
        % h=(Tf-T0)/intsteps;
        Pre_Comp
    case 'chaotic'
        rey=50; % 1/nu
        nu=1/rey;

        % initial/final time
        T0=0;
        Tf=50; 
        
        % initialise time
        t=T0;
        % euler step
        h=.002;
        % inter observation time
        dt_obs=.1;
        % % % number of steps  with plotting
        exsteps=(Tf-T0)/dt_obs;
        % % % number of steps between plotting
        intsteps=dt_obs/h;%ceil((Tf-T0)/exsteps/h);
        Pre_Comp
    case 'test'
         load y_Temp_mcmc_test
%          load mcmc_init State1 State2
%          u1h=State1;
%          u2h=State2;
%         makemult(n,rey,h);
end


if want_plot==1
    if want_video==1
        writerObj = VideoWriter([regime,'3.avi']);
        writerObj.FrameRate = 10;
        open(writerObj);
    end
    % x reduced for quiver plots
    xr=red2d(X,n,np);
    yr=red2d(Y,n,np);
          
    plot_in_loop;
            pause(.1)

end


for j1=1:exsteps % steps till next datapoint
    for j2=1:intsteps % intermediate integration steps
        [u1h,u2h]=ns2dstepaa(u1h,u2h,n,h);
        t=t+h; % anti-alias version
%         if mod(j2,3)==0
             plot_in_loop;
%         end
    end 
    
    if want_plot==1                 
%         plot_in_loop;
%         pause(.1)
    end

end  
if want_video==1
    close(writerObj);
end

toc

