function [temperature,ess,logWeight]=find_temperature(logW,tolerance,Nthresh,N,min_temperature)
% INPUT: resampling threshold Nthresh together with some precision
% tolerance around it and log of un-normalised IS log-weights logW as well as
% number of particles N, min_temperature is current temperature
% e.g. tolerance=1; Nthresh=N/3
% OUTPUT: normalised log-weights, tempering temperature, and effective
% sample size ess
% N. Kantas, Oct 12

% while loop flag
check=1;

% bisection interval
interval=[min_temperature,.5*(min_temperature+1),1];

gg=interval;

% recompute tempered weights
logW_temper=zeros(N,3);
for ii=1:N 
    logW_temper(ii,:)=(logW(ii,1).*(gg-min_temperature)); 
end

% normalise weights
logZ = log(sum(exp(logW_temper-max(max(logW_temper)))))+max(max(logW_temper));
logW_temper2=logW_temper-ones(N,1)*logZ;

% compute ess and ess-Nthresh
GG_ess=1./sum(exp(logW_temper2*2),1);
GG_ess_diff=GG_ess-Nthresh;

% make decisions on whether to accept any points in interval and prepare
% for next iteration of bisection
if GG_ess_diff(3)>=-tolerance
    temperature=1;
    ess=GG_ess(3);
    logWeight=logW_temper2(:,3);
    check=0;    
elseif abs(GG_ess_diff(2))<tolerance
    temperature=gg(2);
    ess=GG_ess(2);
    logWeight=logW_temper2(:,2);
    check=0;    
elseif GG_ess_diff(2)>=0
    interval=[interval(2),(interval(2)+interval(3))/2,interval(3)];
    GG_ess=[GG_ess(2),0,GG_ess(3)];
    GG_ess_diff=[GG_ess_diff(2),0,GG_ess_diff(3)];
elseif  GG_ess_diff(2)<0
    interval=[interval(1),(interval(1)+interval(2))/2,interval(2)];        
    GG_ess=[GG_ess(1),0,GG_ess(2)];
    GG_ess_diff=[GG_ess_diff(1),0,GG_ess_diff(2)];
end

% new candidate midpoint
gg=interval(2);

clear logW_temper logW_temper2;

% bisection starts, repeat as before
counter=1;
while check==1
    % weights for midpoint 
    logW_temper=logW*(gg-min_temperature);
    
    % normalise weights
    logZ = log(sum(exp(logW_temper(:,1)-max(logW_temper(:,1)))))+max(logW_temper(:,1));
    logW_temper2(:,1) = logW_temper(:,1) - logZ;  
    
    % compute ess and ess-Nthresh
    gg_ess=1./sum(exp(logW_temper2*2),1);
    gg_ess2=gg_ess-Nthresh;
    
    % should we return mid point?
    if  abs(gg_ess2)<tolerance
        temperature=gg;
        ess=gg_ess;
        logWeight=logW_temper2;
        check=0;        
    end    
    
    % store in interval     
    GG_ess(2)=gg_ess;
    GG_ess_diff(2)=gg_ess2;
    
    % bisect again
    if GG_ess_diff(2)>=0
        interval=[interval(2),(interval(2)+interval(3))/2,interval(3)];
        GG_ess=[GG_ess(2),0,GG_ess(3)];
        GG_ess_diff=[GG_ess_diff(2),0,GG_ess_diff(3)];
    elseif  GG_ess_diff(2)<0
        interval=[interval(1),(interval(1)+interval(2))/2,interval(2)];
        GG_ess=[GG_ess(1),0,GG_ess(2)];
        GG_ess_diff=[GG_ess_diff(1),0,GG_ess_diff(2)];
    end
    
    % new candidate midpoint
    gg=interval(2);

    counter=counter+1;
end


    