function [ vort_vec ] = get_vort_vec(u1h,u2h,start,fin,half_width)
%Take u- and v- direction of velocities (u1h and u2h) in Fourier space to produce a
%stacked vector with the real and imag parts resp.
%   start and fin are the indices of the low frequency block when in the
%   nxn vorticity matrix, so should be centred on (n/2,n/2) +/- half_width
%   (of a square with edge size of 2*halfwidth +1 )
% N. Kantas Nov 12

global KX KY 
kmod=sqrt(KX.^2 + KY.^2); % matrix of |k| 

% compute 2D Fourier transform
vohh=(KY.*u1h-KX.*u2h)./kmod;

% revert to physical indexing of Fourier modes
voh=fftshift(vohh);

% keep only low frequencies (just half of the box around zero)
vort_low=voh(start:fin,start:fin);


% Block 1 (left columns without column with k1=0)
size1=(half_width^2)*2+half_width;
block1=reshape(vort_low(1:size1),size1,1);
% Block 2 
block2=vort_low(1:half_width,half_width+1);

% % Block 3
% block3=vort_low((half_width+2):end,half_width+1);

vort_vec=[real(block1);
          real(block2);
%           real(block3);
          imag(block1);
          imag(block2);
%           imag(block3)
];

end

