function makemult(n,rey,h)
% generate multiplier matrices for NSscript;
% av,bv for diff in Fourier (dx resp dy)
% mm for time stepping; 
% p1,p2,p3 needed in projection step
% matrices are generated as if FFT was centered, i.e.,
% k=(* -n/2+1 ... -1 | 0 1 ... n/2-1)
% ('0' at position n/2+1, * is unused for even n)
% and then swapped by fftshift to confirm to matlab fft
% such that fftshift(k)=(0 1 ... n/2-1 | * -n/2+1 ... -1)


global av bv mm p1 p2 p3 c L integration_method;

% attention: j1=row-index corresponds to k2=x-Four-coeff.
for j1=1:n % fill av,bv,mm
    k1=j1-n/2-1;
    av(1,j1)=-1i*k1*(2*pi/L);
    bv(j1,1)=-1i*k1*(2*pi/L);
    for j2=1:n
        k2=j2-n/2-1;
        ks=k1*k1+k2*k2;        
        switch lower(integration_method)
            case 'nonlin_explicit'                
                % % use below for linear-implicit non-linear explicit euler scheme
                mm(j1,j2)=1/(1+h*ks/rey);
            case 'etd_1'   
                % % ETD order 1
                mm(j1,j2)=exp(-((2*pi/L)^2)*ks*h/rey);
        end

    end
end

% attention: j1=row-index corresponds to k2=y-Four-coeff.
for j1=1:n % fill p1,p2,p3
    for j2=1:n
        k1=j2-n/2-1;
        k2=j1-n/2-1;
        k1s=k1*k1; 
        k2s=k2*k2; 
        ks=k1s+k2s;
        if (ks==0)
            p1(j1,j2)=0;
            p2(j1,j2)=0;
            p3(j1,j2)=0;
           
        else
            mk=1/ks;
            p1(j1,j2)=mk*k2s;
            p2(j1,j2)=-mk*k1*k2;
            p3(j1,j2)=mk*k1s;%%%%%%%%%%%%% correct
          
        end
    end
end
av=fftshift(av); 
bv=fftshift(bv); 
mm=fftshift(mm);
p1=fftshift(p1);
p2=fftshift(p2); 
p3=fftshift(p3); 

c=(-log(mm)/h);
c(1,1)=1;

end