function [u1,u2]=ns2dstepaa(u1h,u2h,n,h); 
% Antialiased version of euler scheme
% perform integration over dt=h time steps starting at (real) time t,
% u1,u2 are in the x-space (both inputs and outputs) 

global X Y av bv mm w1 w2 w3 w4 g1h g2h c p1 p2 p3 L integration_method; 

% convective nonlinearity
% u \dot \nabla u= [v1h_x+v2h_y,v3h_y+v2h_x]'
% aap2 anti-aliased product (zero padding, iFFT, FFT and low pass times factor)
v1h=aap2(u1h,u1h); 
v2h=aap2(u1h,u2h);
v3h=aap2(u2h,u2h); 

% % comment out for dynamic. forcing
% [g1,g2]=g(X,Y,t); 
% g1h=fft2(g1)/n/n; 
% g2h=fft2(g2)/n/n; 

% differentiate  in Fourier 
for j=1:n                  
    w1(j,:)=av.*v1h(j,:); % loop over x=column index
    w2(:,j)=bv.*v2h(:,j); % loop over y=row index
    w3(j,:)=av.*v2h(j,:); 
    w4(:,j)=bv.*v3h(:,j);
end

r1=-w1-w2+g1h; 
r2=-w3-w4+g2h; 

% % % project forcing and non-linearity to space of periodic,
% % div free, zero mean functions
rr1=(p1.*r1+p2.*r2); 
rr2=(p2.*r1+p3.*r2);

switch lower(integration_method)
    case 'nonlin_explicit'
        % % integrate using linear-implicit non-linear explicit euler scheme (check also makemult.m)
        u1=mm.*(u1h+h*rr1);
        u2=mm.*(u2h+h*rr2);
        
    case 'etd_1'
        % % ETD order
        % integrate using exponential time differencing first order
        u1=mm.*u1h+(ones(size(mm))-mm)./c.*rr1;
        u2=mm.*u2h+(ones(size(mm))-mm)./c.*rr2;       
end


end
