clear
% rng('shuffle')

% SEED=str2num('100');
% rand('twister',SEED)
% randn('state',2^32*rand)

global X Y av bv mm w1 w2 w3 w4 c g1h g2h p1 p2 p3 regime KX KY L integration_method;

% load new data?
loadnew=1;

if loadnew==1
    
    % Use each np-th point in quiver plots;
    np=16;
    
    
    n=64;
    
    L=2*pi;
    
    dx=L/n; dy=dx;
    
    
    integration_method='etd_1';
    regime='chaotic';
    
    switch lower(regime)
        case 'stationary'
            rey=10; % 1/nu
            nu=1/rey;
            
            % initial/final time
            T0=0;
            Tf=40;
            
            % initialise time
            t=T0;
            
            % euler step
            h=.05;
            % inter observation time
            dt_obs=.2;
            % % % number of steps  with plotting
            exsteps=(Tf-T0)/dt_obs;
            % % % number of steps between plotting
            intsteps=dt_obs/h;%ceil((Tf-T0)/exsteps/h);
            % h=(Tf-T0)/intsteps;
        case 'chaotic'
            rey=50; % 1/nu
            nu=1/rey;
            
            % initial/final time
            T0=0;
            Tf=.2;
            
            % initialise time
            t=T0;
            % euler step
            h=.002;
            % inter observation time
            dt_obs=.02;
            % % % number of steps  with plotting
            exsteps=(Tf-T0)/dt_obs;
            % % % number of steps between plotting
            intsteps=dt_obs/h;%ceil((Tf-T0)/exsteps/h);            
    end
    
    x=0:dx:L-dx;
    y=x;
    
    [X,Y]=meshgrid(x,y);
    
    kv=[0:(n/2-1) (-n/2):-1];
    [KX,KY]=meshgrid(kv,kv);
    kmod=sqrt(KX.^2 + KY.^2); % matrix of |k| (for  plotting power spectra)
    
    
    av=zeros(1,n); % av=dx, hence belongs to k2, hence row-vector
    bv=zeros(n,1); % bv=dy, hence belongs to k1, hence column-vect.
    
    % e^ch if exponential multiplier
    mm=zeros(n,n);
    % non-linearities
    w1=zeros(n,n);w2=zeros(n,n);w3=zeros(n,n);w4=zeros(n,n);
    % elements of Phat_k=(p1 p2;p2 p3)
    p1=zeros(n,n); p2=p1; p3=p1;
    % calculating mm p1 p2 p3 w1 to w4
    makemult(n,rey,h); % generate matrices
    
    % % IF FORCING IS TIME-HOMOGENEOUS store g1h,g2h here once
    % comment out for dynamic. forcing
    [g1,g2]=g(X,Y,t);
    g1h=fft2(g1)/n/n;
    g2h=fft2(g2)/n/n;
    
    % ------------------- initialisation -------------
    % initial condition
    
   
    % parameters
    delta=5;
    alph=2.2;
    
    SampleFromPrior
    
    
    gamma=.2;
    observation_type='euler';
    % np here determined how many observations you take     
    [y1,y2]=CreateData(u1h,u2h,n,t,gamma,exsteps,intsteps,h,np,observation_type);
    
    
    save y_Temp_mcmc_test
else

    load y_Temp_mcmc_test
    
end

N=50; % number of particles
particles=cell(N,1);

% particle cells
% x-direction and weights
U1=cell(N,3);
% y-direction
U2=cell(N,2);

% just for tempered weights
U3=cell(N,1);

% sampling from the prior
for ii=1:N
    SampleFromPrior
    % stores intial condition always in 1- and 2- direction
    U1{ii,1}=u1h;
    U2{ii,1}=u2h;
    
    %  stores last state in trajectory (here initialisation)
    U1{ii,2}=u1h;
    U2{ii,2}=u2h;
    
    %  weights
    %  at temperature=1
    U1{ii,3}=[];
    %  at temperature<1
    U3{ii,1}=[];
end
% temp storage for resampling
Utemp1=cell(N,3);
Utemp2=cell(N,2);

ACCEPT_RATIO=cell(1,N);
for l1=1:N
    ACCEPT_RATIO{1,l1}=[];
end
size_grid=length(y1{1})^2;


% for monitoring jitter
MonitorG1real=cell(exsteps,size_grid);
MonitorG1imag=cell(exsteps,size_grid);
MonitorG2real=cell(exsteps,size_grid);
MonitorG2imag=cell(exsteps,size_grid);

MonitorG_TOPreal=cell(exsteps,size_grid);
MonitorG_TOPimag=cell(exsteps,size_grid);
MonitorG_BOTreal=cell(exsteps,size_grid);
MonitorG_BOTimag=cell(exsteps,size_grid);


% mcmc number of steps
num_mcmc_moves=2;
% mcmc step for high frequency
rho=.991;
% mcmc step for low frequency 0.95-.99 for paper
rho1=.96;

% decide on effective sample size below which you resample
Nthresh=N/3;
% tolerance of ESS for finding next temperature
tolerance=1;

% determine low frequency window half width 5-7 here
% adjust rho1 also when this is changed
k_thresh=7;
blk_ind=(n/2+1-k_thresh):(n/2+1+k_thresh);

ESS=[];
for j1=1:exsteps
    for kk=1:size_grid
        temperatures{j1,kk}=[];
    end
end



logWprev=zeros(N,1);
tic
for j1=1:exsteps % steps till next datapoint
    disp(['SMC iteration n=: ' num2str(j1) ' ............................................'])
    % unormalised weight matrix for each smc iteration use just last column
    % for resampling, the rest is needed for path mcmc
    logW=zeros(N,1);

    
    for ii=1:N
        
        % recall stored last velocity field
        u1h=U1{ii,2};
        u2h=U2{ii,2};
        % run solver forward
        for j2=1:intsteps % intermediate integration steps
            [u1h,u2h]=ns2dstepaa(u1h,u2h,n,h);
        end
        % record last velocity field
        U1{ii,2}=u1h;
        U2{ii,2}=u2h;
    end
    
    % in smc sequence target one measurement of from block of observations each time
    for kk=1:size_grid
        
        for ii=1:N
            u1h=U1{ii,2};
            u2h=U2{ii,2};
            logW(ii,1)=log_likeli_k(kk,n,np,u1h,u2h,y1{j1},y2{j1},gamma,observation_type);
            
            %   unormalised log  weights (likelihood)
            U1{ii,3}=[U1{ii,3},logW(ii,1)];
            
            logW(ii,1)=logW(ii,1)+logWprev(ii,1); %%%% check
        end
        
        % copy untemperred weights for tempering 
        for ii=1:N
             U3{ii,1}=U1{ii,3};
        end
        
        temperature=0;
        logWprevTemper=zeros(N,1);
        counter_temp=1;
        
        
        MonitorG_TOPreal{j1,kk}=[];
        MonitorG_TOPimag{j1,kk}=[];
        
        MonitorG_BOTreal{j1,kk}=[];
        MonitorG_BOTimag{j1,kk}=[];
        
        % begin tempering
        while temperature<1
            
            temperature_prev=temperature;
            
            if temperature>0
                
                for ii=1:N
                    u1h=U1{ii,2};
                    u2h=U2{ii,2};
                    logW(ii,1)=log_likeli_k(kk,n,np,u1h,u2h,y1{j1},y2{j1},gamma,observation_type);                    
                end
                
                [temperature,ess,logWeight]=find_temperature(logW,tolerance,Nthresh,N,temperature);
                
                for ii=1:N
                    U3{ii,1}(end)=temperature*U3{ii,1}(end)/temperature_prev;
                end   
                
            else
                
                [temperature,ess,logWeight]=find_temperature(logW,tolerance,Nthresh,N,temperature);
                for ii=1:N
                    U3{ii,1}(end)=temperature*U3{ii,1}(end);
                end                
            end
                                
            % compute mean variance of low frequency components of population 
            % this uses only diagonal covariance. Try get_mu_sigma_test.m for
            % (regularised) matrix with more correlations and adjust rho
            [ mu,Sigma ] = get_mu_sigma(U1,U2,k_thresh,n,N,logWeight);
            
            temperatures{j1,kk}=[temperatures{j1,kk},temperature];
                                   
            disp(['k = ',num2str(kk),'...temperature= ',num2str(temperature),'....ESS: ',num2str(ess),' ..........'])
            ESS=[ESS; ess];
            
            if temperature==1
                
                logWprev=logWeight;
                                
                if ess<=Nthresh
                    Resampling_temper
                end
            else                
                % resample at temperatures<1                 
                Resampling_temper            
            end
            
            MU1big=0;
            for ii=1:N
                MU1big=MU1big+exp(logWeight(ii,1))*U1{ii,1};
            end
                        
            Diff1=cell(num_mcmc_moves,N);
            Diff2=cell(1,N);
            MonitorG2real{j1,kk}{1,1}=0;
            MonitorG2imag{j1,kk}{1,1}=0;
                                   
            MonitorG1real{j1,kk}=cell(1,num_mcmc_moves);
            MonitorG1imag{j1,kk}=cell(1,num_mcmc_moves);
            for ll1=1:num_mcmc_moves
                MonitorG1real{j1,kk}{1,ll1}=0;
                MonitorG1imag{j1,kk}{1,ll1}=0;                 
            end
            
            %             Diff3=cell(1,N);
            %             MonitorG3{j1,kk}{1,1}=0;
            %             MonitorG3{j1,kk}{1,2}=0;
            
            % mcmc move on the path
            for ii=1:N
                % disp(['XXX.......MCMC for particle: ' num2str(ii) ' ...........XXX'])

                mcmc_moves_for_smc_k_temper;
                
                Diff2{1,ii}= exp(logWeight(ii))*(U1{ii,1}-MU1big);
                
                MonitorG2real{j1,kk}{1,1}=MonitorG2real{j1,kk}{1,1}+real(Diff2{1,ii}).^2;
                MonitorG2imag{j1,kk}{1,1}=MonitorG2imag{j1,kk}{1,1}+imag(Diff2{1,ii}).^2;
                
                for ll1=1:m
                    Diff1{ll1,ii}=exp(logWeight(ii))*(Sample{ll1+1,1}-U1{ii,1}); 
                    MonitorG1real{j1,kk}{1,ll1}=MonitorG1real{j1,kk}{1,ll1}+(real(Diff1{ll1,ii}).^2)/2;                
                    MonitorG1imag{j1,kk}{1,ll1}=MonitorG1imag{j1,kk}{1,ll1}+(imag(Diff1{ll1,ii}).^2)/2;
                end           
                
                
                U1{ii,1}=Sample{m+1,1};
                U2{ii,1}=Sample{m+1,2};
                U1{ii,2}=Sample_t{m+1,1};
                U2{ii,2}=Sample_t{m+1,2};
                U3{ii,1}=log_likeli_store{m+1,1};
               
            end
            
%             MU1big_after=0;
%             for ii=1:N
%                 MU1big_after=MU1big_after+exp(logWeight(ii,1))*U1{ii,1};
%             end
%             
%             for ii=1:N
%                 Diff3{1,ii}= exp(logWeight(ii))*(U1{ii,1}-MU1big);
%                 MonitorG3{j1,kk}{1,1}=MonitorG3{j1,kk}{1,1}+real(Diff3{1,ii}).^2;
%                 MonitorG3{j1,kk}{1,2}=MonitorG3{j1,kk}{1,2}+imag(Diff3{1,ii}).^2;
%                 
%             end
            
            MonitorG_TOPreal{j1,kk}=[MonitorG_TOPreal{j1,kk};MonitorG1real{j1,kk}];
            MonitorG_TOPimag{j1,kk}=[MonitorG_TOPimag{j1,kk};MonitorG1imag{j1,kk}];
            
            MonitorG_BOTreal{j1,kk}=[MonitorG_BOTreal{j1,kk};MonitorG2real{j1,kk}];
            MonitorG_BOTimag{j1,kk}=[MonitorG_BOTimag{j1,kk};MonitorG2imag{j1,kk}];
            
            counter_temp=counter_temp+1;
        end
        
       
        
        % copy back tempered weights to untempered
        for ii=1:N
             U1{ii,3}=U3{ii,1};
        end
        
                                        
    end
    
    
    if j1<exsteps
        save(['test_Gdiag72_991_99_',num2str(j1),'.mat'],'-v7.3')
    end
    
end

simtime=toc;

save -v7.3 test_Gdiag72_991_99